/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Stefano Fornari
// @version $Id: Win32ManagementNode.cpp,v 1.10 2005/01/19 10:22:07 magi Exp $
//

#include <stdio.h>  // to be removed
#include <stdlib.h>

/*
 * Win32 implementation of ManagementNode
 */

#include "common/fscapi.h"
#include "spdm/common/Utils.h"
#include "spdm/common/Constants.h"
#include "spdm/common/ManagementNode.h"
#include "spdm/win32/Win32ManagementNode.h"


Win32ManagementNode::Win32ManagementNode(wchar_t* context, wchar_t* name) : ManagementNode(context, name) {
    wsprintf(fullContext, TEXT("Software/%s/%s"), context, name);
    convertSlashes(fullContext);
 }


/*
 * Returns the value of the given property
 *
 * @param property - the property name
 * @param buf - the buffer for the value
 * @param len - the buffer size
 */
void Win32ManagementNode::getPropertyValue(const wchar_t* property, wchar_t* buf, int size) {
    HKEY key = NULL;
    ULONG dim = size;
    DWORD res;	
    long err = 0;

/*
    RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         KEY_READ,
         &key
    );
*/
    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        fullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
    );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }


    err = RegQueryValueEx(
        key,
        property,
        NULL,
        NULL,  // we currently support only strings
        (UCHAR*)buf,
        &dim
    );
    
    if (err == ERROR_SUCCESS) {
        ; //MessageBox(NULL,  TEXT("Correct"), TEXT("getConfigParameter"), MB_OK);
    } else {		
        ;//MessageBox(NULL,  TEXT("Error"), TEXT("getConfigParameter"), MB_OK);
        wsprintf(buf, TEXT(""));
    }

    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}

/*
 * Sets a property value.
 *
 * @param property - the property name
 * @param value - the property value (zero terminated string)
 */
void Win32ManagementNode::setPropertyValue(const wchar_t* property, wchar_t* value) {
    HKEY key;
    DWORD res;
/*
    RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         KEY_READ | KEY_WRITE,
         &key
    );
*/
    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        fullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
    );


    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }

    RegSetValueEx(
        key,
        property,
        NULL,
        REG_SZ,  // we currently support only strings
        (UCHAR*)value,
        (wcslen(value)+1)*sizeof(wchar_t*)
    );


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}


/*
 * delete a key into the Device Manager.
 *
 * @param key - the key to be deleted
 */

int Win32ManagementNode::deleteKey(const wchar_t* keyToDelete) {
    HKEY key;  
    long ret = -1;
    
    ret = RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         0,
         &key
    );    

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }
    
    if (ret != ERROR_SUCCESS) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;

    }
    
    ret = RegDeleteKey(
      key,
      keyToDelete     
    );
    if (ret != ERROR_SUCCESS) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;

    }
    
    
    ret = 0;

    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
    return ret;
}

/*
 * test if the context exists. Return TRUE or FALSE.
 *
 */
BOOL Win32ManagementNode::existsContext() {
    HKEY key;  
    BOOL ret = FALSE;
    
    ret = RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         0,
         &key
    );    

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }
    
    if (ret != ERROR_SUCCESS) {
        ret = FALSE;

    } else {
        ret = TRUE;
    }
        
    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
    return ret;
}

/*
 * Returns this node's children.
 *
 * The ManagementNode objects are created with the new operator and
 * must be discarded by the caller with the operator delete.
 *
 * @param children - the buffer where ManagementNode* must be stored
 * @param size - the size of the children buffer (number of ManagementNode*) in
 *               input; the number of children in output
 */
void Win32ManagementNode::getChildren(ManagementNode** children, int* size) {
    int i;
    ULONG dim = DIM_MANAGEMENT_PATH;
    HKEY key;
    wchar_t child[DIM_MANAGEMENT_PATH];
    wchar_t fullName[DIM_MANAGEMENT_PATH];
    DWORD res;
    HRESULT ret;

/*
    RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         KEY_READ,
         &key
    );
*/
    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        fullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
    );

    //
    // First of all we need to know how many

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }

    getFullName(fullName, DIM_MANAGEMENT_PATH);
    
    for (i=0; i<*size; ++i) {
        dim = DIM_MANAGEMENT_PATH*sizeof(wchar_t);
        child[0] = 0;

        ret = RegEnumKeyEx(key, i, child, &dim, NULL, NULL, NULL, NULL);
        if (ret != ERROR_SUCCESS) {
            if (ret == ERROR_NO_MORE_ITEMS) {
                break;
            } else {
                lastErrorCode = GetLastError();
                wcscpy(lastErrorMsg, TEXT("Error enumerating children nodes"));
                goto finally;
            }
        }
     children[i] = new Win32ManagementNode(fullName, child);
    }
    /*
    for (i=0; i<*size; ++i) {
        if (RegEnumKeyEx(key, i, child, &dim, NULL, NULL, NULL, NULL) == ERROR_NO_MORE_ITEMS) {
            break;
        };

        children[i] = new Win32ManagementNode(fullName, child);
    }
	
    */
    *size = i;


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}

/*
 * Returns how many children belong to this node.
 * A negative number means an error
 *
 */
int Win32ManagementNode::getChildrenCount() {
    HKEY key;
    ULONG howMany = 0;

    RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         KEY_READ,
         &key
    );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);

        goto finally;
    }

    RegQueryInfoKey(
        key, NULL, NULL, NULL, &howMany, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    );


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }

    return howMany;
}

/*
 * Returns a child node of the ManagementNode.
 *
 * The child ManagementNode object is created with the new operator
 * must be discarded by the caller with the operator delete.
 *
 * @param node - subnode name
 */
ManagementNode* Win32ManagementNode::getChild(const wchar_t* node) {
    wchar_t nodeName   [DIM_MANAGEMENT_PATH];
    wchar_t nodeContext[DIM_MANAGEMENT_PATH];
    wchar_t fullName   [DIM_MANAGEMENT_PATH];


    getNodeName(node, nodeName, DIM_MANAGEMENT_PATH);
    getNodeContext(node, nodeContext, DIM_MANAGEMENT_PATH);

    wcscpy(fullName, context);
    wcscat(fullName, TEXT("/"));
    wcscat(fullName, nodeContext);

    return new Win32ManagementNode(fullName, nodeName);
}


/*
 * Device management contexts are expressed as slash '/' separated
 * context path. This function converts the '/' chars in '\\' as
 * accepted by the registry API.
 *
 * @param str - the string to converted
 *
 */
void Win32ManagementNode::convertSlashes(wchar_t* str) {
    int i, l;

    l = wcslen(str);

    for (i=0; i<l; ++i) {
        if (str[i] == L'/') {
            str[i] = L'\\';
        }
    }
}